using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Management;
using System.Text;

using Microsoft.LightSwitch.Designers.ScreenTemplates.Model;
using Microsoft.LightSwitch.Model;
using Microsoft.LightSwitch.Model.Storage;
using Microsoft.LightSwitch.DesignTime;

namespace LsCollapsibleList.ScreenTemplates
{
    internal class LsCollapsibleScreenTemplate : IScreenTemplate
    {
        #region IScreenTemplate Members

        public void Generate(IScreenTemplateHost host)
        {
            // ** Note: This sample code only works for intrinsic LightSwitch data sources **

            // Add a Tab to the screen
            ContentItem tabContentItem =
                host.AddContentItem(host.ScreenTabPagesContentItem, "Group", ContentItemKind.Group);

            // Only proceed if a PrimaryDataSourceProperty was selected
            if (null != host.PrimaryDataSourceProperty)
            {
                #region Interrogate Data Sources
                // Get the ScreenCollectionProperty that was selected by the end user
                ScreenCollectionProperty collectionProperty =
                    (ScreenCollectionProperty)(host.PrimaryDataSourceProperty);

                // Cast the collectionProperty that was selected by the end user to a IDataType
                IDataType primaryDataType =
                    host.FindGlobalModelItem<ISequenceType>(collectionProperty.PropertyType).ElementType as IDataType;

                // Cast the collectionProperty that was selected by the end user to a IEntityType
                IEntityType collectionDataType =
                    host.FindGlobalModelItem<ISequenceType>(collectionProperty.PropertyType).ElementType as IEntityType;

                // Get the entity set of the entity type
                IEntitySetDefinition primaryEntitySet = collectionDataType.Module.GlobalItems
                    .OfType<IEntityContainerDefinition>()
                    .SelectMany(entityContainer => entityContainer.EntitySets)
                    .Single(entitySet => entitySet.EntityType == collectionDataType);

                // Get the name of the "table"
                string PrimaryTableName = primaryEntitySet.Name;

                // Get the related table name -- if there is one
                IDataType relatedDataType = null;
                string RelatedTableName = "";
                string relatedDataTypeName = "";
                if (host.ChildCollectionProperties.Count > 0)
                {
                    // Only look at the first table -- ignore any others
                    ScreenCollectionProperty ChildcollectionProperty =
                        (ScreenCollectionProperty)(host.ChildCollectionProperties[0]);

                    // Cast the ChildcollectionProperty that was selected by the end user to a IDataType
                    relatedDataType =
                        host.FindGlobalModelItem<ISequenceType>(ChildcollectionProperty.PropertyType).ElementType as IDataType;

                    // Get the entity set of the entity type
                    IEntitySetDefinition childEntitySet = relatedDataType.Module.GlobalItems
                        .OfType<IEntityContainerDefinition>()
                        .SelectMany(entityContainer => entityContainer.EntitySets)
                        .Single(entitySet => entitySet.EntityType == relatedDataType);

                    // Get the name of the "table"
                    RelatedTableName = childEntitySet.Name;
                }
                #endregion

                #region Add the query parameters of the data source if needed
                if (null != host.PrimaryDataSourceParameterProperties)
                {
                    // Loop through each parameter of the data source
                    foreach (ScreenPropertyBase parameter in host.PrimaryDataSourceParameterProperties)
                    {
                        // Add the element to the screen so the end user can provide a value
                        // for the parameter
                        ContentItem contentItem =
                            host.AddContentItem(tabContentItem, parameter.Name, parameter);
                    }
                }
                #endregion

                #region AddMainData Button
                if (host.ChildCollectionProperties.Count > 0)
                {
                    ScreenMethod method = host.AddScreenMethod("AddMainData");
                    CallExpression callExpression = host.CreateCallExpression(method.Id);
                    ChainExpression chainExpression = host.CreateChainExpression(callExpression);
                    ScreenExpressionTree expressionTree = new ScreenExpressionTree() { Body = chainExpression };

                    // Create contentItem
                    ContentItem addMainDataContentItem = host.AddContentItem(
                        tabContentItem,
                        "AddMainData",
                        ContentItemKind.Command);
                    host.SetDisplayName(addMainDataContentItem, "Add");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        addMainDataContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        expressionTree);
                }
                #endregion

                #region Add controls to display the data
                ContentItem listContentItem =
                    host.AddContentItem(tabContentItem, primaryDataType.Name, host.PrimaryDataSourceProperty);
                // Set the type to a Tile List
                listContentItem.View = "Microsoft.LightSwitch.MobileWeb:List";

                // Parent Rows Layout
                ContentItem ParentRowsLayoutItem = host.AddContentItem(listContentItem, "ParentRowsLayout", ContentItemKind.Group);

                #region Only show this section if there is a child collection specified
                if (host.ChildCollectionProperties.Count > 0)
                {
                    #region View Model
                    // Add a singular instance of the Primary collection to the View Model
                    var PrimarySingular = host.AddScreenProperty(primaryDataType, primaryDataType.Name);

                    // Add a singular instance of the Related collection to the View Model
                    var RelatedSingular = host.AddScreenProperty(relatedDataType, relatedDataType.Name);
                    relatedDataTypeName = RelatedSingular.Name;
                    #endregion

                    #region Main Screen
                    // Child ColumnsLayout
                    ContentItem ChildColumsLayoutItem = host.AddContentItem(ParentRowsLayoutItem,
                        "ChildColumsLayout", ContentItemKind.Group);
                    ChildColumsLayoutItem.View = "Microsoft.LightSwitch.MobileWeb:ColumnsLayout";
                    // Change the default setting for CompactMargins
                    host.SetControlPropertyValue(ChildColumsLayoutItem,
                        "Microsoft.LightSwitch.MobileWeb:RootGroup", "CompactMargins", "false");

                    #region EditMainData Button
                    if (host.ChildCollectionProperties.Count > 0)
                    {
                        ScreenMethod method = host.AddScreenMethod("EditMainData");
                        CallExpression callExpression = host.CreateCallExpression(method.Id);
                        ChainExpression chainExpression = host.CreateChainExpression(callExpression);
                        ScreenExpressionTree expressionTree = new ScreenExpressionTree() { Body = chainExpression };

                        // Create contentItem
                        ContentItem DataContentItem = host.AddContentItem(
                            ChildColumsLayoutItem,
                            "EditMainData",
                            ContentItemKind.Command);
                        host.SetDisplayName(DataContentItem, "Edit");

                        // Set Tap property to expression tree
                        host.SetControlPropertyExpressionTree(
                            DataContentItem,
                            "Microsoft.LightSwitch.MobileWeb:RootControl",
                            "Tap",
                            expressionTree);
                    }
                    #endregion

                    #region AddData Button
                    if (host.ChildCollectionProperties.Count > 0)
                    {
                        ScreenMethod method = host.AddScreenMethod("AddData");
                        CallExpression callExpression = host.CreateCallExpression(method.Id);
                        ChainExpression chainExpression = host.CreateChainExpression(callExpression);
                        ScreenExpressionTree expressionTree = new ScreenExpressionTree() { Body = chainExpression };

                        // Create contentItem
                        ContentItem DataContentItem = host.AddContentItem(
                            ChildColumsLayoutItem,
                            "AddData",
                            ContentItemKind.Command);
                        host.SetDisplayName(DataContentItem, "Add");

                        // Set Tap property to expression tree
                        host.SetControlPropertyExpressionTree(
                            DataContentItem,
                            "Microsoft.LightSwitch.MobileWeb:RootControl",
                            "Tap",
                            expressionTree);
                    }
                    #endregion

                    // Child RowsLayout
                    ContentItem ChildRowsLayoutItem = host.AddContentItem(ParentRowsLayoutItem,
                        "ChildRowsLayout", ContentItemKind.Group);
                    ChildRowsLayoutItem.View = "Microsoft.LightSwitch.MobileWeb:RowsLayout";

                    // Add the Custom Control to display the nested collection                
                    IEntityPropertyDefinition IdProperty =
                        collectionDataType.Properties.Where(p => p.Name.ToLower() == "id").FirstOrDefault();

                    // Entity of Primary data source must have a Id property to proceeed
                    if (IdProperty != null)
                    {
                        // Expression used to access Id property of the element in the Primary data source
                        ChainExpression nameExpression =
                            host.CreateChainExpression(
                            host.CreateMemberExpression(String.Format(@"{0}/Properties[Id]", collectionDataType.Name))
                            );

                        // Add the CustomControl 
                        ContentItem namePropertyControl = host.AddContentItem(ChildRowsLayoutItem,
                                                                                        "NestedElements",
                                                                                        ContentItemKind.Value,
                                                                                        nameExpression,
                                                                                        IdProperty.PropertyType);

                        namePropertyControl.View = "Microsoft.LightSwitch.MobileWeb:ValueCustomControl";
                    }
                    #endregion

                    #region Popups

                    #region Popup for Entity for primary data source
                    var PrimaryPopup =
                        host.AddContentItem(host.ScreenDialogPagesContentItem,
                        String.Format("{0}Popup", primaryDataType.Name), ContentItemKind.Group);

                    // Add a control to display the Buttons
                    ContentItem PrimaryButtonsContentItem =
                        host.AddContentItem(PrimaryPopup, String.Format("{0} buttons",
                        PrimarySingular.Name), ContentItemKind.Group);
                    PrimaryButtonsContentItem.View = "Microsoft.LightSwitch.MobileWeb:ColumnsLayout";

                    #region SaveDataMain Button
                    ScreenMethod SaveDataMainMethod = host.AddScreenMethod("SaveDataMain");
                    CallExpression SaveDataMainCallExpression = host.CreateCallExpression(SaveDataMainMethod.Id);
                    ChainExpression SaveDataMainChainExpression = host.CreateChainExpression(SaveDataMainCallExpression);
                    ScreenExpressionTree SaveDataMainExpressionTree = new ScreenExpressionTree() { Body = SaveDataMainChainExpression };

                    // Create contentItem
                    ContentItem SaveDataMainContentItem = host.AddContentItem(
                        PrimaryButtonsContentItem,
                        "SaveDataMain",
                        ContentItemKind.Command);
                    host.SetDisplayName(SaveDataMainContentItem, "Save");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        SaveDataMainContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        SaveDataMainExpressionTree);
                    #endregion

                    #region DeleteDataMain Button
                    ScreenMethod DeleteDataMainMethod = host.AddScreenMethod("DeleteDataMain");
                    CallExpression DeleteDataMainCallExpression = host.CreateCallExpression(DeleteDataMainMethod.Id);
                    ChainExpression DeleteDataMainChainExpression = host.CreateChainExpression(DeleteDataMainCallExpression);
                    ScreenExpressionTree DeleteDataMainExpressionTree = new ScreenExpressionTree() { Body = DeleteDataMainChainExpression };

                    // Create contentItem
                    ContentItem DeleteDataMainContentItem = host.AddContentItem(
                        PrimaryButtonsContentItem,
                        "DeleteDataMain",
                        ContentItemKind.Command);
                    host.SetDisplayName(DeleteDataMainContentItem, "Delete");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        DeleteDataMainContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        DeleteDataMainExpressionTree);
                    #endregion

                    #region CancelData Button
                    ScreenMethod CancelDataMethod = host.AddScreenMethod("CancelData");
                    CallExpression CancelDataCallExpression = host.CreateCallExpression(CancelDataMethod.Id);
                    ChainExpression CancelDataChainExpression = host.CreateChainExpression(CancelDataCallExpression);
                    ScreenExpressionTree CancelDataExpressionTree = new ScreenExpressionTree() { Body = CancelDataChainExpression };

                    // Create contentItem
                    ContentItem CancelDataMainContentItem = host.AddContentItem(
                        PrimaryButtonsContentItem,
                        "CancelData",
                        ContentItemKind.Command);
                    host.SetDisplayName(CancelDataMainContentItem, "Cancel");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        CancelDataMainContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        CancelDataExpressionTree);
                    #endregion

                    // Add a control to display the data
                    ContentItem PrimaryPopupContentItem =
                        host.AddContentItem(PrimaryPopup, PrimarySingular.Name, PrimarySingular);
                    PrimaryPopupContentItem.View = "Microsoft.LightSwitch.MobileWeb:RowsLayout";
                    // Call ExpandContentItem to create elements to represent fields in the Entities
                    // disableNavigationPropertyGeneration - indicates if navigation to an associated table is shown
                    // disableGeneratedPropertyGeneration - indicates if automatic fields (like the audit fields) are shown
                    host.ExpandContentItem(PrimaryPopupContentItem, disableNavigationPropertyGeneration: false,
                        disableGeneratedPropertyGeneration: false);
                    #endregion

                    #region Popup for Entity for Child Collection
                    var RelatedPopup =
                        host.AddContentItem(host.ScreenDialogPagesContentItem, String.Format("{0}Popup",
                        relatedDataType.Name), ContentItemKind.Group);

                    // Add a control to display the Buttons
                    ContentItem RelatedButtonsContentItem =
                        host.AddContentItem(RelatedPopup, String.Format("{0} buttons", RelatedSingular.Name),
                        ContentItemKind.Group);
                    RelatedButtonsContentItem.View = "Microsoft.LightSwitch.MobileWeb:ColumnsLayout";

                    #region SaveData Button
                    ScreenMethod SaveDataMethod = host.AddScreenMethod("SaveData");
                    CallExpression SaveDataCallExpression = host.CreateCallExpression(SaveDataMethod.Id);
                    ChainExpression SaveDataChainExpression = host.CreateChainExpression(SaveDataCallExpression);
                    ScreenExpressionTree SaveDataExpressionTree = new ScreenExpressionTree() { Body = SaveDataChainExpression };

                    // Create contentItem
                    ContentItem SaveDataContentItem = host.AddContentItem(
                        RelatedButtonsContentItem,
                        "SaveData",
                        ContentItemKind.Command);
                    host.SetDisplayName(SaveDataContentItem, "Save");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        SaveDataContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        SaveDataExpressionTree);
                    #endregion

                    #region DeleteData Button
                    ScreenMethod DeleteDataMethod = host.AddScreenMethod("DeleteData");
                    CallExpression DeleteDataCallExpression = host.CreateCallExpression(DeleteDataMethod.Id);
                    ChainExpression DeleteDataChainExpression = host.CreateChainExpression(DeleteDataCallExpression);
                    ScreenExpressionTree DeleteDataExpressionTree = new ScreenExpressionTree() { Body = DeleteDataChainExpression };

                    // Create contentItem
                    ContentItem DeleteDataContentItem = host.AddContentItem(
                        RelatedButtonsContentItem,
                        "DeleteData",
                        ContentItemKind.Command);
                    host.SetDisplayName(DeleteDataContentItem, "Delete");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        DeleteDataContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        DeleteDataExpressionTree);
                    #endregion

                    #region CancelData Button
                    // This reuses the CancelData method created for the Primary Popup

                    // Create contentItem
                    ContentItem CancelDataContentItem = host.AddContentItem(
                        RelatedButtonsContentItem,
                        "CancelData",
                        ContentItemKind.Command);
                    host.SetDisplayName(CancelDataContentItem, "Cancel");

                    // Set Tap property to expression tree
                    host.SetControlPropertyExpressionTree(
                        CancelDataContentItem,
                        "Microsoft.LightSwitch.MobileWeb:RootControl",
                        "Tap",
                        CancelDataExpressionTree);
                    #endregion

                    // Add a control to display the data                    
                    ContentItem RelatedPopupContentItem =
                        host.AddContentItem(RelatedPopup, RelatedSingular.Name, RelatedSingular);
                    RelatedPopupContentItem.View = "Microsoft.LightSwitch.MobileWeb:RowsLayout";
                    // Call ExpandContentItem to create elements to represent fields in the Entities                    
                    host.ExpandContentItem(RelatedPopupContentItem, disableNavigationPropertyGeneration: false,
                        disableGeneratedPropertyGeneration: false);
                    #endregion

                    #endregion
                }
                #endregion

                #endregion

                #region Create JavaScript / Methods / Buttons
                if (host.ChildCollectionProperties.Count > 0)
                {
                    #region Create JavaScript
                    StringBuilder sb = new StringBuilder();

                    #region Default JavaScript for the header
                    sb.Clear();
                    sb.Append("{0}var _screen;");
                    sb.Append("{0}var groupCounter = 0;");
                    sb.Append("{0}var _element = new Array();");
                    sb.Append("{0}var _int{4}Id = new Array();");
                    sb.Append("{0}var SelectedEntity;");
                    sb.Append("{0}");
                    sb.Append("{0}myapp.{1}.created = function (screen) {{");
                    sb.Append("{0}    // Cache screen");
                    sb.Append("{0}    _screen = screen;");
                    sb.Append("{0}    // Center Popups");
                    sb.Append("{0}    $(window).one('popupcreate', function (e) {{");
                    sb.Append("{0}        $(e.target).popup({{");
                    sb.Append("{0}            positionTo: 'window'");
                    sb.Append("{0}        }});");
                    sb.Append("{0}    }});");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name
                        ));
                    #endregion

                    #region JavaScript for the _postRender for the List control
                    sb.Clear();
                    sb.Append("{0}myapp.{1}.ParentRowsLayout_postRender = function (element, contentItem) {{");
                    sb.Append("{0}    // Get Section Name");
                    sb.Append("{0}    // Get Summary Property");
                    sb.Append("{0}    var entityType = contentItem.value.details.entity.details.getModel();");
                    sb.Append("{0}    var summaryAttribute = getAttributeForEntity_{1}(entityType, ':@SummaryProperty');");
                    sb.Append("{0}    var SummaryProperty = summaryAttribute.property.name;");
                    sb.Append("{0}    var SectionName = eval('contentItem.value.' + SummaryProperty);");
                    sb.Append("{0}    // Get Section Id");
                    sb.Append("{0}    var intSectionId = contentItem.value.Id;");
                    sb.Append("{0}");
                    sb.Append("{0}    // Wrap contents in a collapsible section");
                    sb.Append("{0}    if (myapp.LastSectionOpened == undefined) {{");
                    sb.Append("{0}        myapp.LastSectionOpened = 0;");
                    sb.Append("{0}    }};");
                    sb.Append("{0}    var option;");
                    sb.Append("{0}    if (myapp.LastSectionOpened == intSectionId) {{");
                    sb.Append("{0}        // If the last Section was the one that was opened Open it");
                    sb.Append("{0}        option = {{ collapsed: false }};");
                    sb.Append("{0}    }} else {{");
                    sb.Append("{0}        option = {{ collapsed: true }};");
                    sb.Append("{0}    }}");
                    sb.Append("{0}");
                    sb.Append("{0}    collapsibleContent_{1}(element, contentItem, SectionName, option);");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName
                        ));
                    #endregion

                    #region JavaScript for NestedItems Custom Control
                    sb.Clear();
                    sb.Append("{0}myapp.{1}.NestedElements_render = function (element, contentItem) {{");
                    sb.Append("{0}    // Get the Id of the {4}");
                    sb.Append("{0}    var int{4}Id = contentItem.value;");
                    sb.Append("{0}    // Save values for reload later");
                    sb.Append("{0}    _int{4}Id.splice(groupCounter, 0, int{4}Id);");
                    sb.Append("{0}    _element.splice(groupCounter, 0, element);");
                    sb.Append("{0}    // Increase groupCounter");
                    sb.Append("{0}    groupCounter = groupCounter + 1;");
                    sb.Append("{0}    // Load ");
                    sb.Append("{0}    getElementsForSection_{1}(int{4}Id, element);");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name
                        ));
                    #endregion

                    #region JavaScript for collapsibleContent method
                    sb.Clear();
                    sb.Append("{0}function collapsibleContent_{1}(element, contentItem, groupTitle, options) {{");
                    sb.Append("{0}    // From:");
                    sb.Append("{0}    // jQuery Mobile Collapsible Content Control with LightSwitch	");
                    sb.Append("{0}    // http://jewellambert.com/jquery-mobile-collapsible-content-control-with-lightswitch/");
                    sb.Append("{0}    // Jewel Lambert - @dotnetlore");
                    sb.Append("{0}");
                    sb.Append("{0}    // provide some defaults for the optional 'options' parameter");
                    sb.Append("{0}    var defaults = {{ dataTheme: 'b', contentTheme: 'b', collapsed: false }};");
                    sb.Append("{0}    options = $.extend({{}}, defaults, options);");
                    sb.Append("{0}");
                    sb.Append("{0}    // create a header based on the displayName of the bound content");
                    sb.Append("{0}    var h1 = $('<h1>').text(groupTitle);");
                    sb.Append("{0}");
                    sb.Append("{0}    $(element).prepend(h1);");
                    sb.Append("{0}");
                    sb.Append("{0}    // build the <div> for the jQM collapsible content control");
                    sb.Append("{0}    var DIV = $('<div data-role=collapsible data-theme=a data-collapsed=' + options.collapsed + ' />');");
                    sb.Append("{0}");
                    sb.Append("{0}    // wrap the existing children with this div");
                    sb.Append("{0}    $(element).children().wrapAll(DIV);");
                    sb.Append("{0}");
                    sb.Append("{0}    // tell jQM to render the new <div>");
                    sb.Append("{0}    DIV.trigger('create');");
                    sb.Append("{0}}}");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName
                        ));
                    #endregion

                    #region JavaScript for getElementsForSection
                    sb.Clear();
                    sb.Append("{0}function getElementsForSection_{1}({4}Id, element) {{");
                    sb.Append("{0}    // clear element");
                    sb.Append("{0}    $(element).empty();");
                    sb.Append("{0}");
                    sb.Append("{0}    // Get Elements");
                    sb.Append("{0}    var Int32 = ':Int32';");
                    sb.Append("{0}    var filter = '({4}/Id eq ' + msls._toODataString({4}Id, Int32) + ')';");
                    sb.Append("{0}    msls.showProgress(myapp.activeDataWorkspace.ApplicationData.{3}");
                    sb.Append("{0}    .filter(filter)");
                    sb.Append("{0}    .execute()");
                    sb.Append("{0}    .then(function (result) {{");
                    sb.Append("{0}");
                    sb.Append("{0}        // Make an array of buttons");
                    sb.Append("{0}        var objButton = new Array();");
                    sb.Append("{0}        // Create a JQuery Mobile container");
                    sb.Append("{0}        var objFieldcontain = $('<div data-role=fieldcontain />');");
                    sb.Append("{0}        var objFieldset = $('<fieldset data-role=controlgroup />');");
                    sb.Append("{0}        objFieldset.appendTo($(objFieldcontain));");
                    sb.Append("{0}        var CustomUl = $('<ul class=msls-listview ui-listview data-role=listview data-inset=false></ul>');");
                    sb.Append("{0}        CustomUl.appendTo($(objFieldset));");
                    sb.Append("{0}");
                    sb.Append("{0}        // Loop through each page");
                    sb.Append("{0}        var index = 0;");
                    sb.Append("{0}        var Entities = result.results;");
                    sb.Append("{0}        Entities.forEach(function (entity) {{");
                    sb.Append("{0}            index = index + 1;");
                    sb.Append("{0}            // Create Button text");
                    sb.Append("{0}");
                    sb.Append("{0}            // Get Summary Property");
                    sb.Append("{0}            var entityType = entity.details.getModel();");
                    sb.Append("{0}            var summaryAttribute = getAttributeForEntity_{1}(entityType, ':@SummaryProperty');");
                    sb.Append("{0}            var SummaryProperty = summaryAttribute.property.name;");
                    sb.Append("{0}");
                    sb.Append("{0}            var shortText = eval('entity.' + SummaryProperty);");
                    sb.Append("{0}");
                    sb.Append("{0}            // Trim text if it is long");
                    sb.Append("{0}            if (shortText.length > 200) {{");
                    sb.Append("{0}                shortText = jQuery.trim(shortText)");
                    sb.Append("{0}                    .substring(0, 200).split(' ').slice(0, -1).join(' ') + '...';");
                    sb.Append("{0}            }}");
                    sb.Append("{0}");
                    sb.Append("{0}            var CustomDiv = '<li tabindex=0 data-msls=true ';");
                    sb.Append("{0}");
                    sb.Append("{0}            CustomDiv = CustomDiv + 'onclick=edit{3}_{1}(' + entity.Id + ',' + {4}Id + '); ';");
                    sb.Append("{0}            CustomDiv = CustomDiv + 'return=false rel=external';");
                    sb.Append("{0}            CustomDiv = CustomDiv + '>';");
                    sb.Append("{0}");
                    sb.Append("{0}            CustomDiv = CustomDiv + '<div class=msls-text-container>';");
                    sb.Append("{0}            CustomDiv = CustomDiv + '<label class=msls-label-text>' + shortText + '</label>';");
                    sb.Append("{0}            CustomDiv = CustomDiv + '</div>';");
                    sb.Append("{0}");
                    sb.Append("{0}            CustomDiv = CustomDiv + '</div>';");
                    sb.Append("{0}            CustomDiv = CustomDiv + '<div class=msls-clear></div>';");
                    sb.Append("{0}");
                    sb.Append("{0}            CustomDiv = CustomDiv + '</li>';");
                    sb.Append("{0}");
                    sb.Append("{0}            // Create the Button");
                    sb.Append("{0}            objButton[index] = $(CustomDiv);");
                    sb.Append("{0}");
                    sb.Append("{0}            // Add Div to the CustomUl");
                    sb.Append("{0}            objButton[index].appendTo($(CustomUl));");
                    sb.Append("{0}        }})");
                    sb.Append("{0}");
                    sb.Append("{0}        // Add contaner to the element");
                    sb.Append("{0}        objFieldset.appendTo($(element));");
                    sb.Append("{0}        // Tell JQuery Mobile to render");
                    sb.Append("{0}        objFieldset.trigger('create');");
                    sb.Append("{0}    }}))");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name
                        ));
                    #endregion

                    #region JavaScript for buttons and events
                    sb.Clear();
                    sb.Append("{0}function edit{3}_{1}(entityId, {4}Id ) {{");
                    sb.Append("{0}    // Create a filter");
                    sb.Append("{0}    var int32 = ':Int32';");
                    sb.Append("{0}    var filter = '(Id eq ' + msls._toODataString(entityId, int32) + ')';");
                    sb.Append("{0}    // Use filter in query to get the page");
                    sb.Append("{0}    myapp.activeDataWorkspace.ApplicationData.{3}");
                    sb.Append("{0}        .filter(filter)");
                    sb.Append("{0}        .execute()");
                    sb.Append("{0}        .then(function (result) {{");
                    sb.Append("{0}            SelectedEntity = result.results[0];");
                    sb.Append("{0}            _screen.{6} = SelectedEntity;");
                    sb.Append("{0}            _screen.showPopup('{5}Popup');");
                    sb.Append("{0}        }});");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name,
                        relatedDataTypeName
                        ));

                    sb.Clear();
                    sb.Append("{0}// {2}");
                    sb.Append("{0}");
                    sb.Append("{0}myapp.{1}.SaveDataMain_execute = function (screen) {{");
                    sb.Append("{0}    SaveMainChanges_{1}();");
                    sb.Append("{0}}};");
                    sb.Append("{0}myapp.{1}.AddMainData_execute = function (screen) {{");
                    sb.Append("{0}    // Create A New Record");
                    sb.Append("{0}    screen.{4} = new myapp.{4}();");
                    sb.Append("{0}    screen.showPopup('{4}Popup');");
                    sb.Append("{0}}};");
                    sb.Append("{0}myapp.{1}.DeleteDataMain_execute = function (screen) {{");
                    sb.Append("{0}    // Delete");
                    sb.Append("{0}    screen.{4}.deleteEntity();");
                    sb.Append("{0}    SaveMainChanges_{1}();");
                    sb.Append("{0}}};");
                    sb.Append("{0}myapp.{1}.EditMainData_execute = function (screen) {{");
                    sb.Append("{0}    // Show the Popup");
                    sb.Append("{0}    screen.{4} = screen.{2}.selectedItem;");
                    sb.Append("{0}    screen.showPopup('{4}Popup');");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name
                        ));

                    sb.Clear();
                    sb.Append("{0}// {3}");
                    sb.Append("{0}");
                    sb.Append("{0}myapp.{1}.SaveData_execute = function (screen) {{");
                    sb.Append("{0}    SaveChanges_{1}();");
                    sb.Append("{0}}};");
                    sb.Append("{0}myapp.{1}.AddData_execute = function (screen) {{");
                    sb.Append("{0}    // Create A New Record");
                    sb.Append("{0}    SelectedEntity = new myapp.{5}();");
                    sb.Append("{0}    SelectedEntity.set{4}(screen.{2}.selectedItem);");
                    sb.Append("{0}    _screen.{6} = SelectedEntity;");
                    sb.Append("{0}    _screen.showPopup('{5}Popup');");
                    sb.Append("{0}}};");
                    sb.Append("{0}myapp.{1}.DeleteData_execute = function (screen) {{");
                    sb.Append("{0}    // Delete");
                    sb.Append("{0}    SelectedEntity.deleteEntity();");
                    sb.Append("{0}    SaveChanges_{1}();");
                    sb.Append("{0}}};");
                    sb.Append("{0}myapp.{1}.CancelData_execute = function (screen) {{");
                    sb.Append("{0}    // Discard Changes");
                    sb.Append("{0}    screen.details.dataWorkspace.ApplicationData.details.discardChanges();");
                    sb.Append("{0}    // Close Popup");
                    sb.Append("{0}    _screen.closePopup();");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name,
                        relatedDataTypeName
                        ));

                    sb.Clear();
                    sb.Append("{0}// Save");
                    sb.Append("{0}");
                    sb.Append("{0}function SaveMainChanges_{1}() {{");
                    sb.Append("{0}    // Save changes");
                    sb.Append("{0}    myapp.applyChanges().then(function () {{");
                    sb.Append("{0}        // Close Popup");
                    sb.Append("{0}        _screen.closePopup();");
                    sb.Append("{0}        // Reload");
                    sb.Append("{0}        _screen.{2}.load();");
                    sb.Append("{0}    }}, function fail(e) {{");
                    sb.Append("{0}        // Show the error.");
                    sb.Append("{0}        msls.showMessageBox(e.message, {{ title: 'Error' }}).then(function () {{");
                    sb.Append("{0}            // Cancel Changes");
                    sb.Append("{0}            myapp.cancelChanges();");
                    sb.Append("{0}            // Close Popup");
                    sb.Append("{0}            _screen.closePopup();");
                    sb.Append("{0}            // Reload");
                    sb.Append("{0}            _screen.{2}.load();");
                    sb.Append("{0}        }});");
                    sb.Append("{0}    }});");
                    sb.Append("{0}}}");
                    sb.Append("{0}function SaveChanges_{1}() {{");
                    sb.Append("{0}    // Save changes");
                    sb.Append("{0}    myapp.applyChanges().then(function () {{");
                    sb.Append("{0}        // Close Popup");
                    sb.Append("{0}        _screen.closePopup();");
                    sb.Append("{0}        // Load all sections");
                    sb.Append("{0}        for (var i = 0; i < groupCounter; i++) {{");
                    sb.Append("{0}            // Load the Section ");
                    sb.Append("{0}            getElementsForSection_{1}(_int{4}Id[i], _element[i]);");
                    sb.Append("{0}        }}");
                    sb.Append("{0}    }}, function fail(e) {{");
                    sb.Append("{0}        // Show the error.");
                    sb.Append("{0}        msls.showMessageBox(e, {{ title: 'Error' }}).then(function () {{");
                    sb.Append("{0}            // Discard Changes");
                    sb.Append("{0}            screen.details.dataWorkspace.ApplicationData.details.discardChanges();");
                    sb.Append("{0}        }});");
                    sb.Append("{0}    }});");
                    sb.Append("{0}}}");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName,
                        PrimaryTableName,
                        RelatedTableName,
                        primaryDataType.Name,
                        relatedDataType.Name
                        ));
                    #endregion

                    #region JavaScript getAttributeForEntity Utility Function
                    sb.Clear();
                    sb.Append("{0}// Utility");
                    sb.Append("{0}");
                    sb.Append("{0}function getAttributeForEntity_{1}(item, classId) {{");
                    sb.Append("{0}");
                    sb.Append("{0}    var att = null,");
                    sb.Append("{0}        s = null;");
                    sb.Append("{0}    if (item) {{");
                    sb.Append("{0}        att = item[classId];");
                    sb.Append("{0}        if (att) {{");
                    sb.Append("{0}            if ($.isArray(att)) {{");
                    sb.Append("{0}                if (att.length > 0) {{");
                    sb.Append("{0}                    s = att[0];");
                    sb.Append("{0}                }}");
                    sb.Append("{0}            }} else {{");
                    sb.Append("{0}                s = att;");
                    sb.Append("{0}            }}");
                    sb.Append("{0}        }}");
                    sb.Append("{0}    }}");
                    sb.Append("{0}    return s;");
                    sb.Append("{0}}};");

                    // Call AddScreenCodeBehind to render JavaScript
                    host.AddScreenCodeBehind(String.Format(sb.ToString(),
                        Environment.NewLine,
                        host.ScreenName));
                    #endregion

                    #endregion
                }
                #endregion
            }
        }
        #endregion

        #region IScreenTemplateMetadata Members

        public string Description
        {
            get { return "Creates a Collapsible List"; }
        }

        public string DisplayName
        {
            get { return "Ls Collapsible Screen"; }
        }

        public Uri PreviewImage
        {
            get { return new Uri("/LsCollapsibleList.Design;component/Resources/ScreenTemplateImages/LsCollapsibleScreenTemplateLarge.png", UriKind.Relative); }
        }

        public RootDataSourceType RootDataSource
        {
            get { return RootDataSourceType.Collection; }
        }

        public string ScreenNameFormat
        {
            get { return "{0}LsCollapsibleScreen"; }
        }

        public Uri SmallIcon
        {
            get { return new Uri("/LsCollapsibleList.Design;component/Resources/ScreenTemplateImages/LsCollapsibleScreenTemplateSmall.png", UriKind.Relative); }
        }

        public bool SupportsChildCollections
        {
            get { return true; }
        }

        public string TemplateName
        {
            get { return LsCollapsibleScreenTemplate.TemplateId; }
        }

        #endregion

        #region Constants

        internal const string TemplateId = "LsCollapsibleList:LsCollapsibleScreenTemplate";

        #endregion
    }


    [Export(typeof(IScreenTemplateFactory))]
    [Template(LsCollapsibleScreenTemplate.TemplateId, ScreenEditMode.Browse)]
    [TargetPlatform(TargetPlatformNames.MobileWeb)]

    internal class LsCollapsibleScreenTemplateFactory : IScreenTemplateFactory
    {
        #region IScreenTemplateFactory Members

        IScreenTemplate IScreenTemplateFactory.CreateScreenTemplate()
        {
            return new LsCollapsibleScreenTemplate();
        }

        #endregion
    }
}